@@ -15,6 +15,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vfsops.c,v 1.42.4.14 2007/02/09 01:23:40 shadow Exp $");
 
 #define __NO_VERSION__		/* don't define kernel_version in module.h */
 #include <linux/module.h> /* early to avoid printf->printk mapping */
@@ -28,7 +30,6 @@
 #include "h/smp_lock.h"
 #endif
 
-#include "osi_compat.h"
 
 struct vcache *afs_globalVp = 0;
 struct vfs *afs_globalVFS = 0;
@@ -38,6 +39,9 @@ struct vfsmount *afs_cacheMnt;
 int afs_was_mounted = 0;	/* Used to force reload if mount/unmount/mount */
 
 extern struct super_operations afs_sops;
+#if defined(AFS_LINUX26_ENV)
+extern struct export_operations afs_export_ops;
+#endif
 extern afs_rwlock_t afs_xvcache;
 extern struct afs_q VLRU;
 
@@ -101,7 +105,9 @@ struct file_system_type afs_fs_type = {
 #endif
 
 #if defined(AFS_LINUX26_ENV)
-struct backing_dev_info *afs_backing_dev_info;
+struct backing_dev_info afs_backing_dev_info = {
+    .ra_pages		= 0, /* disable readahead, afs does prefetch */
+};
 
 int
 afs_fill_super(struct super_block *sb, void *data, int silent)
@@ -140,24 +146,8 @@ afs_read_super(struct super_block *sb, void *data, int silent)
     sb->s_blocksize_bits = 10;
     sb->s_magic = AFS_VFSMAGIC;
     sb->s_op = &afs_sops;	/* Super block (vfs) ops */
-
-#if defined(STRUCT_SUPER_BLOCK_HAS_S_D_OP)
-    sb->s_d_op = &afs_dentry_operations;
-#endif
-
-    /* used for inodes backing_dev_info field, also */
-    afs_backing_dev_info = osi_Alloc(sizeof(struct backing_dev_info));
-#if defined(HAVE_BDI_INIT)
-    bdi_init(afs_backing_dev_info);
-#endif
-#if defined(STRUCT_BDI_HAS_NAME)
-    afs_backing_dev_info->name = "openafs";
-#endif
-    afs_backing_dev_info->ra_pages = 0;
-#if defined (STRUCT_SUPER_BLOCK_HAS_S_BDI)
-    sb->s_bdi = afs_backing_dev_info;
-    /* The name specified here will appear in the flushing thread name - flush-afs */
-    bdi_register(afs_backing_dev_info, NULL, "afs");
+#if defined(AFS_LINUX26_ENV)
+    sb->s_export_op = &afs_export_ops;
 #endif
 #if defined(MAX_NON_LFS)
 #ifdef AFS_64BIT_CLIENT
@@ -176,7 +166,6 @@ afs_read_super(struct super_block *sb, void *data, int silent)
     code = afs_root(sb);
     if (code) {
 	afs_globalVFS = NULL;
-	osi_linux_free_inode_pages();
 #if defined(AFS_LINUX26_ENV)
         module_put(THIS_MODULE);
 #else
@@ -232,9 +221,7 @@ afs_root(struct super_block *afsp)
 #else
 		afsp->s_root = d_alloc_root(ip, NULL);
 #endif
-#if !defined(STRUCT_SUPER_BLOCK_HAS_S_D_OP)
 		afsp->s_root->d_op = &afs_dentry_operations;
-#endif
 	    } else
 		code = ENOENT;
 	}
@@ -282,14 +269,18 @@ afs_notify_change(struct dentry *dp, struct iattr *iattrp)
 
 
 #if defined(STRUCT_SUPER_HAS_ALLOC_INODE)
-static afs_kmem_cache_t *afs_inode_cachep;
+static kmem_cache_t *afs_inode_cachep;
 
 static struct inode *
 afs_alloc_inode(struct super_block *sb)
 {
     struct vcache *vcp;
 
-    vcp = (struct vcache *) kmem_cache_alloc(afs_inode_cachep, KALLOC_TYPE);
+#if defined(SLAB_KERNEL)
+    vcp = (struct vcache *) kmem_cache_alloc(afs_inode_cachep, SLAB_KERNEL);
+#else
+    vcp = (struct vcache *) kmem_cache_alloc(afs_inode_cachep, GFP_KERNEL);
+#endif
     if (!vcp)
 	return NULL;
 
@@ -302,26 +293,27 @@ afs_destroy_inode(struct inode *inode)
     kmem_cache_free(afs_inode_cachep, inode);
 }
 
-void
-init_once(void * foo)
+static void
+init_once(void * foo, kmem_cache_t * cachep, unsigned long flags)
 {
     struct vcache *vcp = (struct vcache *) foo;
 
-    inode_init_once(AFSTOV(vcp));
+    if ((flags & (SLAB_CTOR_VERIFY|SLAB_CTOR_CONSTRUCTOR)) ==
+	SLAB_CTOR_CONSTRUCTOR)
+	inode_init_once(AFSTOV(vcp));
 }
 
 int
 afs_init_inodecache(void)
 {
-#if defined(KMEM_CACHE_TAKES_DTOR)
-    afs_inode_cachep = kmem_cache_create("afs_inode_cache",
-		sizeof(struct vcache), 0,
-		SLAB_HWCACHE_ALIGN | SLAB_RECLAIM_ACCOUNT, init_once_func, NULL);
-#else
-    afs_inode_cachep = kmem_cache_create("afs_inode_cache",
-		sizeof(struct vcache), 0,
-		SLAB_HWCACHE_ALIGN | SLAB_RECLAIM_ACCOUNT, init_once_func);
+#ifndef SLAB_RECLAIM_ACCOUNT
+#define SLAB_RECLAIM_ACCOUNT 0
 #endif
+
+    afs_inode_cachep = kmem_cache_create("afs_inode_cache",
+					 sizeof(struct vcache),
+					 0, SLAB_HWCACHE_ALIGN | SLAB_RECLAIM_ACCOUNT,
+					 init_once, NULL);
     if (afs_inode_cachep == NULL)
 	return -ENOMEM;
     return 0;
@@ -330,8 +322,7 @@ afs_init_inodecache(void)
 void
 afs_destroy_inodecache(void)
 {
-    if (afs_inode_cachep)
-	(void) kmem_cache_destroy(afs_inode_cachep);
+    (void) kmem_cache_destroy(afs_inode_cachep);
 }
 #else
 int
@@ -347,25 +338,6 @@ afs_destroy_inodecache(void)
 }
 #endif
 
-#if defined(STRUCT_SUPER_OPERATIONS_HAS_EVICT_INODE)
-static void
-afs_evict_inode(struct inode *ip)
-{
-    struct vcache *vcp = VTOAFS(ip);
-
-    if (vcp->vlruq.prev || vcp->vlruq.next)
-	osi_Panic("inode freed while on LRU");
-    if (vcp->hnext)
-	osi_Panic("inode freed while still hashed");
-
-    truncate_inode_pages(&ip->i_data, 0);
-    end_writeback(ip);
-
-#if !defined(STRUCT_SUPER_HAS_ALLOC_INODE)
-    afs_osi_Free(ip->u.generic_ip, sizeof(struct vcache));
-#endif
-}
-#else
 static void
 afs_clear_inode(struct inode *ip)
 {
@@ -380,7 +352,6 @@ afs_clear_inode(struct inode *ip)
     afs_osi_Free(ip->u.generic_ip, sizeof(struct vcache));
 #endif
 }
-#endif
 
 /* afs_put_super
  * Called from unmount to release super_block. */
@@ -407,10 +378,6 @@ afs_put_super(struct super_block *sbp)
 #endif
 
     osi_linux_verify_alloced_memory();
-#if defined(HAVE_BDI_INIT)
-    bdi_destroy(afs_backing_dev_info);
-#endif
-    osi_Free(afs_backing_dev_info, sizeof(struct backing_dev_info));
     AFS_GUNLOCK();
 
     sbp->s_dev = 0;
@@ -479,11 +446,7 @@ struct super_operations afs_sops = {
   .alloc_inode =	afs_alloc_inode,
   .destroy_inode =	afs_destroy_inode,
 #endif
-#if defined(STRUCT_SUPER_OPERATIONS_HAS_EVICT_INODE)
-  .evict_inode =	afs_evict_inode,
-#else
   .clear_inode =	afs_clear_inode,
-#endif
   .put_super =		afs_put_super,
   .statfs =		afs_statfs,
 #if !defined(AFS_LINUX24_ENV)
@@ -553,17 +516,43 @@ vattr2inode(struct inode *ip, struct vattr *vp)
     ip->i_mode = vp->va_mode;
     ip->i_uid = vp->va_uid;
     ip->i_gid = vp->va_gid;
-    i_size_write(ip, vp->va_size);
+    ip->i_size = vp->va_size;
 #if defined(AFS_LINUX26_ENV)
     ip->i_atime.tv_sec = vp->va_atime.tv_sec;
-    ip->i_atime.tv_nsec = 0;
     ip->i_mtime.tv_sec = vp->va_mtime.tv_sec;
-    ip->i_mtime.tv_nsec = 0;
+    /* Set the mtime nanoseconds to the sysname generation number.
+     * This convinces NFS clients that all directories have changed
+     * any time the sysname list changes.
+     */
+    ip->i_mtime.tv_nsec = afs_sysnamegen;
     ip->i_ctime.tv_sec = vp->va_ctime.tv_sec;
-    ip->i_ctime.tv_nsec = 0;
 #else
     ip->i_atime = vp->va_atime.tv_sec;
     ip->i_mtime = vp->va_mtime.tv_sec;
     ip->i_ctime = vp->va_ctime.tv_sec;
 #endif
 }
+
+/* osi_linux_free_inode_pages
+ *
+ * Free all vnodes remaining in the afs hash.  Must be done before
+ * shutting down afs and freeing all memory.
+ */
+void
+osi_linux_free_inode_pages(void)
+{
+    int i;
+    struct vcache *tvc, *nvc;
+    extern struct vcache *afs_vhashT[VCSIZE];
+
+    for (i = 0; i < VCSIZE; i++) {
+	for (tvc = afs_vhashT[i]; tvc; ) {
+	    int slept;
+	
+	    nvc = tvc->hnext;
+	    if (afs_FlushVCache(tvc, &slept))		/* slept always 0 for linux? */
+		printf("Failed to invalidate all pages on inode 0x%p\n", tvc);
+	    tvc = nvc;
+	}
+    }
+}
